package eg;

import static org.seamframework.sql.Expressions.and;
import static org.seamframework.sql.Expressions.eq;
import static org.seamframework.sql.Expressions.parameter;

import javax.enterprise.event.Observes;
import javax.inject.Inject;

import org.jboss.weld.environment.se.events.ContainerInitialized;
import org.seamframework.sql.Column;
import org.seamframework.sql.Create;
import org.seamframework.sql.ExecutableStatement;
import org.seamframework.sql.From;
import org.seamframework.sql.Insert;
import org.seamframework.sql.Parameter;
import org.seamframework.sql.Query;
import org.seamframework.sql.Selection;
import org.seamframework.sql.Session;
import org.seamframework.sql.Table;
import org.seamframework.sql.Tuple;
import org.seamframework.sql.Update;

public class Main {
	
	static class users {
		static final Table<users> table = new Table<users>("users");
		static final Column<users,String> name = new Column<users, String>("name", String.class);
		static final Column<users,String> username = new Column<users, String>("username", String.class);
		static final Column<users,String> password = new Column<users, String>("password", String.class);
		static final Column<users,Integer> age = new Column<users, Integer>("age", Integer.class);
	}

	static class permissions {
		static final Table<permissions> table = new Table<permissions>("permissions");
		static final Column<permissions, String> username = new Column<permissions, String>("username", String.class);
		static final Column<permissions, String> id = new Column<permissions, String>("id", String.class);
	}
		
	void start(@Observes ContainerInitialized ci, Session session) {
		createTable(session);
		testInsert(session);
		testUpdate(session);
		testLoginQuery(session);
		testQuery2(session);
	}
	
	/*public static void main(String[] args) throws Exception {
		ConnectionFactory cf = new TestConnectionFactory();
		PreparedStatementFactory psf = new DefaultPreparedStatementFactory();
		Session session = new Session( new AnnotationLiteral<Default>() {}, cf, psf );
		
		createTable(session);
		testInsert(session);
		testUpdate(session);
		testQuery(session);
	}*/

	public static void createTable(Session session) {
		Create<users> create = new Create<users>(users.table);
		create.add(users.name, "varchar not null");
		create.add(users.username, "varchar not null primary key");
		create.add(users.password, "varchar not null");
		create.add(users.age, "integer not null");
		System.out.println( create.toSql() );
		
		session.prepare(create).execute();

		Create<permissions> create2 = new Create<permissions>(permissions.table);
		create2.add(permissions.username, "varchar not null");
		create2.add(permissions.id, "varchar not null primary key");
		System.out.println( create2.toSql() );
		
		session.prepare(create2).execute();
}

	public static void testInsert(Session session) {
		Insert<users> insert = new Insert<users>(users.table);
		Parameter<String> nameParameter = insert.set(users.name);
		Parameter<String> usernameParameter = insert.set(users.username);
		Parameter<String> passwordParameter = insert.set(users.password);
		Parameter<Integer> ageParameter = insert.set(users.age);
		System.out.println( insert.toSql() );
		
		ExecutableStatement es = session.prepare(insert);
		es.set(nameParameter, "Gavin King");
		es.set(usernameParameter, "gavin");
		es.set(passwordParameter, "foobar");
		es.set(ageParameter, 21);
		es.execute();

		Insert<permissions> insert2 = new Insert<permissions>(permissions.table);
		Parameter<String> userParameter = insert2.set(permissions.username);
		Parameter<String> idParameter = insert2.set(permissions.id);
		System.out.println( insert.toSql() );
		
		ExecutableStatement es2 = session.prepare(insert2);
		es2.set(userParameter, "gavin");
		es2.set(idParameter, "create");
		es2.execute();
		es2.set(idParameter, "update");
		es2.execute();
		es2.set(idParameter, "delete");
		es2.execute();
}

	public static void testUpdate(Session session) {
		Update<users> update = new Update<users>(users.table);
		Parameter<String> nameParameter = update.set(users.name);
		Parameter<String> passwordParameter = update.set(users.password);
		Parameter<String> usernameParam = parameter(String.class);
		update.where( eq( update.get(users.username), usernameParam ) );
		System.out.println( update.toSql() );
		
		ExecutableStatement es = session.prepare(update);
		es.set(nameParameter, "Gavin King");
		es.set(passwordParameter, "foobax");
		es.set(usernameParam, "gavin");
		es.execute();
	}
	
	
	@Inject LoginQuery loginQuery;

	public void testLoginQuery(Session session) {
		
		System.out.println( loginQuery.toSql() );
		
		Iterable<Tuple> result = session.prepare(loginQuery)
			.set(loginQuery.username, "gavin")
			.set(loginQuery.password, "foobax")
			.result();
		
		for (Tuple tuple: result) {
			System.out.println( tuple.get(loginQuery.name) + ' ' + 
						tuple.get(loginQuery.age) + ' ' + 
						tuple.get(loginQuery.permissionId) );
		}
	}

	public static void testQuery2(Session session) {
		
		Query query = new Query();
		From<users> user = query.from(users.table, "u");

		Parameter<String> username = parameter(String.class);
		Parameter<String> password = parameter(String.class);
		
		query.where( and( eq( user.get(users.username), username ), 
				          eq( user.get(users.password), password ) ) );

		Selection<String> name = query.select( user.get(users.name), "n" );
		Selection<Integer> age = query.select( user.get(users.age), "a" );

		query.orderBy(name, age);
		
		System.out.println( query.toSql() );
		
		Iterable<Tuple> result = session.prepare(query)
			.set(username, "gavin")
			.set(password, "foobax")
			.result();
		
		for (Tuple tuple: result) {
			System.out.println( tuple.get(name) + ' ' + tuple.get(age) );
		}
	}

}
