package org.seamframework.sql;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

public class ExecutableStatement {
	
	private final List<Parameter<?>> parameters;
	private final PreparedStatement ps;
	
	public ExecutableStatement(Statement statement, Session session) {
		parameters = statement.getParameters();
		ps = session.createStatement(statement.toSql(), session.getQualifier());
	}
	
	public PreparedStatement getPreparedStatement() {
		return ps;
	}

	public <T> ExecutableStatement set(Parameter<T> param, T value) {
		try {
			int i = parameters.indexOf(param) + 1;
			if (i<0) throw new IllegalArgumentException();
			if (value==null) {
				ps.setNull(i, getSqlType(param.getType()));
			}
			else {
				ps.setObject(i, value);
			}
			return this;
		}
		catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
	}
	
	public int execute() {
		try {
			return ps.executeUpdate();
		}
		catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
	}
	
	private int getSqlType(Class<?> javaType) {
		//TODO:
		return Types.VARCHAR;
	}
}
