package org.seamframework.sql;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class Expressions {
	
	public static <T> Expression<Boolean> eq(final Expression<T> x, final Expression<T> y) {
		return new Expression<Boolean>() {
			@Override
			public String toSql() {
				return new StringBuffer()
					.append( x.toSql() )
					.append('=')
					.append( y.toSql() )
					.toString();
			}
			@Override
			public Class<Boolean> getType() {
				return Boolean.class;
			}
			@Override
			public List<Parameter<?>> getParameters() {
				List<Parameter<?>> params = new ArrayList<Parameter<?>>();
				params.addAll(x.getParameters());
				params.addAll(y.getParameters());
				return params;
			}
		};
	}
	
	public static <T> Parameter<T> parameter(final Class<T> type) {
		return new Parameter<T>() {
			@Override
			public Class<T> getType() {
				return type;
			}
			@Override
			public String toSql() {
				return "?";
			}
			@Override
			public List<Parameter<?>> getParameters() {
				return Collections.<Parameter<?>>singletonList(this);
			}
		};
	}
	
	public static Expression<Boolean> and(final Expression<Boolean>... expressions) {
		return new Expression<Boolean>() {
			@Override
			public String toSql() {
				StringBuilder builder = new StringBuilder();
				builder.append('(');
				for (Iterator<Expression<Boolean>> i = Arrays.asList(expressions).iterator(); i.hasNext(); ) {
					builder.append( i.next().toSql() );
					if ( i.hasNext() ) builder.append(" and ");
				}
				builder.append(')');
				return builder.toString();
			}
			@Override
			public Class<Boolean> getType() {
				return Boolean.class;
			}
			@Override
			public List<Parameter<?>> getParameters() {
				List<Parameter<?>> params = new ArrayList<Parameter<?>>();
				for (Expression<Boolean> e: expressions) {
					params.addAll(e.getParameters());
				}
				return params;
			}
		};
	}
	
}
